@@ -1,10 +1,18 @@
 // Copyright The OpenTelemetry Authors
 // SPDX-License-Identifier: Apache-2.0
 
+#include <memory>
+#include <utility>
+
 #include "opentelemetry/exporters/ostream/span_exporter_factory.h"
+#include "opentelemetry/sdk/trace/processor.h"
+#include "opentelemetry/sdk/trace/recordable.h"
 #include "opentelemetry/sdk/trace/simple_processor_factory.h"
+#include "opentelemetry/sdk/trace/tracer_provider.h"
 #include "opentelemetry/sdk/trace/tracer_provider_factory.h"
 #include "opentelemetry/trace/provider.h"
+#include "opentelemetry/trace/span_id.h"
+#include "opentelemetry/trace/tracer_provider.h"
 
 #ifdef BAZEL_BUILD
 #  include "examples/common/foo_library/foo_library.h"
@@ -18,22 +26,32 @@ namespace trace_exporter = opentelemetry::exporter::trace;
 
 namespace
 {
-void initTracer()
+void InitTracer()
 {
   auto exporter  = trace_exporter::OStreamSpanExporterFactory::Create();
   auto processor = trace_sdk::SimpleSpanProcessorFactory::Create(std::move(exporter));
-  std::shared_ptr<opentelemetry::trace::TracerProvider> provider =
+
+  std::shared_ptr<opentelemetry::sdk::trace::TracerProvider> sdk_provider =
       trace_sdk::TracerProviderFactory::Create(std::move(processor));
 
   // Set the global trace provider
-  trace_api::Provider::SetTracerProvider(provider);
+  const std::shared_ptr<opentelemetry::trace::TracerProvider> &api_provider = sdk_provider;
+  trace_api::Provider::SetTracerProvider(api_provider);
+}
+
+void CleanupTracer()
+{
+  std::shared_ptr<opentelemetry::trace::TracerProvider> noop;
+  trace_api::Provider::SetTracerProvider(noop);
 }
 }  // namespace
 
 int main()
 {
   // Removing this line will leave the default noop TracerProvider in place.
-  initTracer();
+  InitTracer();
 
   foo_library();
+
+  CleanupTracer();
 }
