@@ -3,6 +3,8 @@
 
 #include "opentelemetry/exporters/ostream/span_exporter_factory.h"
 #include "opentelemetry/exporters/ostream/span_exporter.h"
+#include "opentelemetry/sdk/trace/recordable.h"
+#include "opentelemetry/version.h"
 
 namespace trace_sdk = opentelemetry::sdk::trace;
 
@@ -15,7 +17,7 @@ namespace trace
 std::unique_ptr<trace_sdk::SpanExporter> OStreamSpanExporterFactory::Create()
 {
   return Create(std::cout);
-};
+}
 
 std::unique_ptr<trace_sdk::SpanExporter> OStreamSpanExporterFactory::Create(std::ostream &sout)
 {
