@@ -7,6 +7,7 @@
 
 #include "opentelemetry/exporters/otlp/otlp_http_exporter_options.h"
 #include "opentelemetry/sdk/trace/exporter.h"
+#include "opentelemetry/version.h"
 
 OPENTELEMETRY_BEGIN_NAMESPACE
 namespace exporter
@@ -17,7 +18,7 @@ namespace otlp
 /**
  * Factory class for OtlpHttpExporter.
  */
-class OtlpHttpExporterFactory
+class OPENTELEMETRY_EXPORT OtlpHttpExporterFactory
 {
 public:
   /**
