@@ -1,7 +1,22 @@
 // Copyright The OpenTelemetry Authors
 // SPDX-License-Identifier: Apache-2.0
 
-#include "opentelemetry/exporters/otlp/otlp_http_client.h"
+#include <algorithm>
+#include <atomic>
+#include <chrono>
+#include <condition_variable>
+#include <cstddef>
+#include <functional>
+#include <iostream>
+#include <list>
+#include <map>
+#include <memory>
+#include <mutex>
+#include <ratio>
+#include <string>
+#include <unordered_map>
+#include <utility>
+#include <vector>
 
 #if defined(HAVE_GSL)
 #  include <gsl/gsl>
@@ -9,49 +24,35 @@
 #  include <assert.h>
 #endif
 
+#include "nlohmann/json.hpp"
+#include "opentelemetry/common/timestamp.h"
+#include "opentelemetry/exporters/otlp/otlp_environment.h"
+#include "opentelemetry/exporters/otlp/otlp_http.h"
+#include "opentelemetry/exporters/otlp/otlp_http_client.h"
+#include "opentelemetry/ext/http/client/http_client.h"
 #include "opentelemetry/ext/http/client/http_client_factory.h"
 #include "opentelemetry/ext/http/common/url_parser.h"
-
-#include "opentelemetry/exporters/otlp/protobuf_include_prefix.h"
-
-#include "google/protobuf/message.h"
-#include "google/protobuf/reflection.h"
-#include "google/protobuf/stubs/common.h"
-#include "google/protobuf/stubs/stringpiece.h"
-#include "nlohmann/json.hpp"
-
-#if defined(GOOGLE_PROTOBUF_VERSION) && GOOGLE_PROTOBUF_VERSION >= 3007000
-#  include "google/protobuf/stubs/strutil.h"
-#else
-#  include "google/protobuf/stubs/port.h"
-namespace google
-{
-namespace protobuf
-{
-LIBPROTOBUF_EXPORT void Base64Escape(StringPiece src, std::string *dest);
-}  // namespace protobuf
-}  // namespace google
-#endif
-
-#include "opentelemetry/exporters/otlp/protobuf_include_suffix.h"
-
-#include "opentelemetry/common/timestamp.h"
+#include "opentelemetry/nostd/string_view.h"
+#include "opentelemetry/nostd/variant.h"
+#include "opentelemetry/sdk/common/base64.h"
+#include "opentelemetry/sdk/common/exporter_utils.h"
 #include "opentelemetry/sdk/common/global_log_handler.h"
-#include "opentelemetry/sdk_config.h"
+#include "opentelemetry/version.h"
 
-#include <atomic>
-#include <condition_variable>
-#include <fstream>
-#include <mutex>
-#include <sstream>
-#include <string>
-#include <vector>
+// clang-format off
+#include "opentelemetry/exporters/otlp/protobuf_include_prefix.h" // IWYU pragma: keep
+// clang-format on
+#include <google/protobuf/descriptor.h>
+#include <google/protobuf/message.h>
+#include <google/protobuf/stubs/port.h>
+// clang-format off
+#include "opentelemetry/exporters/otlp/protobuf_include_suffix.h" // IWYU pragma: keep
+// clang-format on
 
 #ifdef GetMessage
 #  undef GetMessage
 #endif
 
-namespace nostd       = opentelemetry::nostd;
 namespace http_client = opentelemetry::ext::http::client;
 
 OPENTELEMETRY_BEGIN_NAMESPACE
@@ -64,7 +65,7 @@ namespace
 {
 
 /**
- * This class handles the response message from the Elasticsearch request
+ * This class handles the response message from the HTTP request
  */
 class ResponseHandler : public http_client::EventHandler
 {
@@ -75,8 +76,21 @@ public:
   ResponseHandler(std::function<bool(opentelemetry::sdk::common::ExportResult)> &&callback,
                   bool console_debug = false)
       : result_callback_{std::move(callback)}, console_debug_{console_debug}
+  {}
+
+  std::string BuildResponseLogMessage(http_client::Response &response,
+                                      const std::string &body) noexcept
   {
-    stopping_.store(false);
+    std::stringstream ss;
+    ss << "Status:" << response.GetStatusCode() << ", Header:";
+    response.ForEachHeader([&ss](opentelemetry::nostd::string_view header_name,
+                                 opentelemetry::nostd::string_view header_value) {
+      ss << "\t" << header_name.data() << ": " << header_value.data() << ",";
+      return true;
+    });
+    ss << "Body:" << body;
+
+    return ss.str();
   }
 
   /**
@@ -85,6 +99,8 @@ public:
    */
   void OnResponse(http_client::Response &response) noexcept override
   {
+    sdk::common::ExportResult result = sdk::common::ExportResult::kSuccess;
+    std::string log_message;
     // Lock the private members so they can't be read while being modified
     {
       std::unique_lock<std::mutex> lk(mutex_);
@@ -92,17 +108,20 @@ public:
       // Store the body of the request
       body_ = std::string(response.GetBody().begin(), response.GetBody().end());
 
-      if (console_debug_)
+      if (!(response.GetStatusCode() >= 200 && response.GetStatusCode() <= 299))
       {
-        std::stringstream ss;
-        ss << "[OTLP HTTP Client] Status:" << response.GetStatusCode() << "Header:";
-        response.ForEachHeader([&ss](opentelemetry::nostd::string_view header_name,
-                                     opentelemetry::nostd::string_view header_value) {
-          ss << "\t" << header_name.data() << " : " << header_value.data() << ",";
-          return true;
-        });
-        ss << "Body:" << body_;
-        OTEL_INTERNAL_LOG_DEBUG(ss.str());
+        log_message = BuildResponseLogMessage(response, body_);
+
+        OTEL_INTERNAL_LOG_ERROR("[OTLP HTTP Client] Export failed, " << log_message);
+        result = sdk::common::ExportResult::kFailure;
+      }
+      else if (console_debug_)
+      {
+        if (log_message.empty())
+        {
+          log_message = BuildResponseLogMessage(response, body_);
+        }
+        OTEL_INTERNAL_LOG_DEBUG("[OTLP HTTP Client] Export success, " << log_message);
       }
     }
 
@@ -110,7 +129,7 @@ public:
       bool expected = false;
       if (stopping_.compare_exchange_strong(expected, true, std::memory_order_release))
       {
-        Unbind(sdk::common::ExportResult::kSuccess);
+        Unbind(result);
       }
     }
   }
@@ -269,7 +288,7 @@ public:
       case http_client::SessionState::WriteError:
         if (console_debug_)
         {
-          OTEL_INTERNAL_LOG_DEBUG("[OTLP HTTP Client] DEBUG:Session state: error writing request");
+          OTEL_INTERNAL_LOG_DEBUG("[OTLP HTTP Client] Session state: error writing request");
         }
         break;
 
@@ -325,7 +344,7 @@ public:
   {
     session_ = &session;
     owner_   = owner;
-  };
+  }
 
 private:
   // Define a mutex to keep thread safety
@@ -336,7 +355,7 @@ private:
   const opentelemetry::ext::http::client::Session *session_ = nullptr;
 
   // Whether notify has been called
-  std::atomic<bool> stopping_;
+  std::atomic<bool> stopping_{false};
 
   // A string to store the response body
   std::string body_ = "";
@@ -393,16 +412,12 @@ static std::string BytesMapping(const std::string &bytes,
       }
       else
       {
-        std::string base64_value;
-        google::protobuf::Base64Escape(bytes, &base64_value);
-        return base64_value;
+        return opentelemetry::sdk::common::Base64Escape(bytes);
       }
     }
     case JsonBytesMappingKind::kBase64: {
       // Base64 is the default bytes mapping of protobuf
-      std::string base64_value;
-      google::protobuf::Base64Escape(bytes, &base64_value);
-      return base64_value;
+      return opentelemetry::sdk::common::Base64Escape(bytes);
     }
     case JsonBytesMappingKind::kHex:
       return HexEncode(bytes);
@@ -421,6 +436,7 @@ static void ConvertListFieldToJson(nlohmann::json &value,
                                    const google::protobuf::FieldDescriptor *field_descriptor,
                                    const OtlpHttpClientOptions &options);
 
+// NOLINTBEGIN(misc-no-recursion)
 static void ConvertGenericMessageToJson(nlohmann::json &value,
                                         const google::protobuf::Message &message,
                                         const OtlpHttpClientOptions &options)
@@ -431,7 +447,7 @@ static void ConvertGenericMessageToJson(nlohmann::json &value,
   {
     const google::protobuf::FieldDescriptor *field_descriptor = fields_with_data[i];
     nlohmann::json &child_value = options.use_json_name ? value[field_descriptor->json_name()]
-                                                        : value[field_descriptor->name()];
+                                                        : value[field_descriptor->camelcase_name()];
     if (field_descriptor->is_repeated())
     {
       ConvertListFieldToJson(child_value, message, field_descriptor, options);
@@ -641,10 +657,17 @@ void ConvertListFieldToJson(nlohmann::json &value,
   }
 }
 
+// NOLINTEND(misc-no-recursion) suppressing for performance, if implemented iterative process needs
+// Dynamic memory allocation
+
 }  // namespace
 
 OtlpHttpClient::OtlpHttpClient(OtlpHttpClientOptions &&options)
-    : is_shutdown_(false), options_(options), http_client_(http_client::HttpClientFactory::Create())
+    : is_shutdown_(false),
+      options_(options),
+      http_client_(http_client::HttpClientFactory::Create()),
+      start_session_counter_(0),
+      finished_session_counter_(0)
 {
   http_client_->SetMaxSessionsPerConnection(options_.max_requests_per_connection);
 }
@@ -683,7 +706,11 @@ OtlpHttpClient::~OtlpHttpClient()
 
 OtlpHttpClient::OtlpHttpClient(OtlpHttpClientOptions &&options,
                                std::shared_ptr<ext::http::client::HttpClient> http_client)
-    : is_shutdown_(false), options_(options), http_client_(http_client)
+    : is_shutdown_(false),
+      options_(std::move(options)),
+      http_client_(std::move(http_client)),
+      start_session_counter_(0),
+      finished_session_counter_(0)
 {
   http_client_->SetMaxSessionsPerConnection(options_.max_requests_per_connection);
 }
@@ -692,12 +719,13 @@ OtlpHttpClient::OtlpHttpClient(OtlpHttpClientOptions &&options,
 opentelemetry::sdk::common::ExportResult OtlpHttpClient::Export(
     const google::protobuf::Message &message) noexcept
 {
-  opentelemetry::sdk::common::ExportResult session_result =
-      opentelemetry::sdk::common::ExportResult::kSuccess;
+  std::shared_ptr<opentelemetry::sdk::common::ExportResult> session_result =
+      std::make_shared<opentelemetry::sdk::common::ExportResult>(
+          opentelemetry::sdk::common::ExportResult::kSuccess);
   opentelemetry::sdk::common::ExportResult export_result = Export(
       message,
-      [&session_result](opentelemetry::sdk::common::ExportResult result) {
-        session_result = result;
+      [session_result](opentelemetry::sdk::common::ExportResult result) {
+        *session_result = result;
         return result == opentelemetry::sdk::common::ExportResult::kSuccess;
       },
       0);
@@ -707,7 +735,7 @@ opentelemetry::sdk::common::ExportResult OtlpHttpClient::Export(
     return export_result;
   }
 
-  return session_result;
+  return *session_result;
 }
 
 sdk::common::ExportResult OtlpHttpClient::Export(
@@ -740,7 +768,7 @@ sdk::common::ExportResult OtlpHttpClient::Export(
   if (options_.console_debug)
   {
     OTEL_INTERNAL_LOG_DEBUG(
-        "[OTLP HTTP Client] DEBUG: Waiting for response from "
+        "[OTLP HTTP Client] Waiting for response from "
         << options_.url << " (timeout = "
         << std::chrono::duration_cast<std::chrono::milliseconds>(options_.timeout).count()
         << " milliseconds)");
@@ -773,52 +801,64 @@ bool OtlpHttpClient::ForceFlush(std::chrono::microseconds timeout) noexcept
 
   // Wait for all the sessions to finish
   std::unique_lock<std::mutex> lock(session_waker_lock_);
-  if (timeout <= std::chrono::microseconds::zero())
+
+  std::chrono::steady_clock::duration timeout_steady =
+      std::chrono::duration_cast<std::chrono::steady_clock::duration>(timeout);
+  if (timeout_steady <= std::chrono::steady_clock::duration::zero())
+  {
+    timeout_steady = (std::chrono::steady_clock::duration::max)();
+  }
+
+  size_t wait_counter = start_session_counter_.load(std::memory_order_acquire);
+
+  while (timeout_steady > std::chrono::steady_clock::duration::zero())
   {
-    while (true)
     {
+      std::lock_guard<std::recursive_mutex> guard{session_manager_lock_};
+      if (running_sessions_.empty())
       {
-        std::lock_guard<std::recursive_mutex> guard{session_manager_lock_};
-        if (running_sessions_.empty())
-        {
-          break;
-        }
-      }
-      // When changes of running_sessions_ and notify_one/notify_all happen between predicate
-      // checking and waiting, we should not wait forever.We should cleanup gc sessions here as soon
-      // as possible to call FinishSession() and cleanup resources.
-      if (std::cv_status::timeout == session_waker_.wait_for(lock, options_.timeout))
-      {
-        cleanupGCSessions();
+        break;
       }
     }
-    return true;
-  }
-  else
-  {
-    return session_waker_.wait_for(lock, timeout, [this] {
-      std::lock_guard<std::recursive_mutex> guard{session_manager_lock_};
-      return running_sessions_.empty();
-    });
+    // When changes of running_sessions_ and notify_one/notify_all happen between predicate
+    // checking and waiting, we should not wait forever.We should cleanup gc sessions here as soon
+    // as possible to call FinishSession() and cleanup resources.
+    std::chrono::steady_clock::time_point start_timepoint = std::chrono::steady_clock::now();
+    if (std::cv_status::timeout == session_waker_.wait_for(lock, options_.timeout))
+    {
+      cleanupGCSessions();
+    }
+    else if (finished_session_counter_.load(std::memory_order_acquire) >= wait_counter)
+    {
+      break;
+    }
+
+    timeout_steady -= std::chrono::steady_clock::now() - start_timepoint;
   }
+
+  return timeout_steady > std::chrono::steady_clock::duration::zero();
 }
 
 bool OtlpHttpClient::Shutdown(std::chrono::microseconds timeout) noexcept
 {
+  is_shutdown_.store(true, std::memory_order_release);
+
+  bool force_flush_result = ForceFlush(timeout);
+
   {
     std::lock_guard<std::recursive_mutex> guard{session_manager_lock_};
-    is_shutdown_ = true;
 
     // Shutdown the session manager
     http_client_->CancelAllSessions();
     http_client_->FinishAllSessions();
   }
 
-  ForceFlush(timeout);
-
+  // Wait util all sessions are canceled.
   while (cleanupGCSessions())
-    ;
-  return true;
+  {
+    ForceFlush(std::chrono::milliseconds{1});
+  }
+  return force_flush_result;
 }
 
 void OtlpHttpClient::ReleaseSession(
@@ -826,20 +866,21 @@ void OtlpHttpClient::ReleaseSession(
 {
   bool has_session = false;
 
+  std::lock_guard<std::recursive_mutex> guard{session_manager_lock_};
+
+  auto session_iter = running_sessions_.find(&session);
+  if (session_iter != running_sessions_.end())
   {
-    std::lock_guard<std::recursive_mutex> guard{session_manager_lock_};
+    // Move session and handle into gc list, and they will be destroyed later
+    gc_sessions_.emplace_back(std::move(session_iter->second));
+    running_sessions_.erase(session_iter);
 
-    auto session_iter = running_sessions_.find(&session);
-    if (session_iter != running_sessions_.end())
-    {
-      // Move session and handle into gc list, and they will be destroyed later
-      gc_sessions_.emplace_back(std::move(session_iter->second));
-      running_sessions_.erase(session_iter);
-
-      has_session = true;
-    }
+    finished_session_counter_.fetch_add(1, std::memory_order_release);
+    has_session = true;
   }
 
+  // Call session_waker_.notify_all() with session_manager_lock_ locked to keep session_waker_
+  // available when destroying OtlpHttpClient
   if (has_session)
   {
     session_waker_.notify_all();
@@ -861,7 +902,7 @@ OtlpHttpClient::createSession(
       std::string error_message = "[OTLP HTTP Client] Export failed, invalid url: " + options_.url;
       if (options_.console_debug)
       {
-        std::cerr << error_message << std::endl;
+        std::cerr << error_message << '\n';
       }
       OTEL_INTERNAL_LOG_ERROR(error_message.c_str());
 
@@ -926,7 +967,7 @@ OtlpHttpClient::createSession(
     const char *error_message = "[OTLP HTTP Client] Export failed, exporter is shutdown";
     if (options_.console_debug)
     {
-      std::cerr << error_message << std::endl;
+      std::cerr << error_message << '\n';
     }
     OTEL_INTERNAL_LOG_ERROR(error_message);
 
@@ -938,13 +979,21 @@ OtlpHttpClient::createSession(
 
   for (auto &header : options_.http_headers)
   {
-    request->AddHeader(header.first, header.second);
+    request->AddHeader(header.first,
+                       opentelemetry::ext::http::common::UrlDecoder::Decode(header.second));
   }
   request->SetUri(http_uri_);
+  request->SetSslOptions(options_.ssl_options);
   request->SetTimeoutMs(std::chrono::duration_cast<std::chrono::milliseconds>(options_.timeout));
   request->SetMethod(http_client::Method::Post);
   request->SetBody(body_vec);
   request->ReplaceHeader("Content-Type", content_type);
+  request->ReplaceHeader("User-Agent", options_.user_agent);
+
+  if (options_.compression == "gzip")
+  {
+    request->SetCompression(opentelemetry::ext::http::client::Compression::kGzip);
+  }
 
   // Returns the created session data
   return HttpSessionData{
@@ -960,16 +1009,20 @@ void OtlpHttpClient::addSession(HttpSessionData &&session_data) noexcept
     return;
   }
 
-  opentelemetry::ext::http::client::Session *key = session_data.session.get();
-  ResponseHandler *handle = static_cast<ResponseHandler *>(session_data.event_handle.get());
+  std::shared_ptr<opentelemetry::ext::http::client::Session> session = session_data.session;
+  std::shared_ptr<opentelemetry::ext::http::client::EventHandler> handle =
+      session_data.event_handle;
+  {
+    std::lock_guard<std::recursive_mutex> guard{session_manager_lock_};
+    static_cast<ResponseHandler *>(handle.get())->Bind(this, *session);
 
-  handle->Bind(this, *key);
-
-  HttpSessionData &store_session_data = running_sessions_[key];
-  store_session_data                  = std::move(session_data);
+    HttpSessionData &store_session_data = running_sessions_[session.get()];
+    store_session_data                  = std::move(session_data);
+  }
 
+  start_session_counter_.fetch_add(1, std::memory_order_release);
   // Send request after the session is added
-  key->SendRequest(store_session_data.event_handle);
+  session->SendRequest(handle);
 }
 
 bool OtlpHttpClient::cleanupGCSessions() noexcept
@@ -992,7 +1045,7 @@ bool OtlpHttpClient::cleanupGCSessions() noexcept
 
 bool OtlpHttpClient::IsShutdown() const noexcept
 {
-  return is_shutdown_;
+  return is_shutdown_.load(std::memory_order_acquire);
 }
 
 }  // namespace otlp
