@@ -1,7 +1,7 @@
 // Copyright The OpenTelemetry Authors
 // SPDX-License-Identifier: Apache-2.0
 
-#ifndef HAVE_CPP_STDLIB
+#ifndef OPENTELEMETRY_STL_VERSION
 
 #  include <chrono>
 #  include <thread>
@@ -15,12 +15,15 @@
 #  include "opentelemetry/exporters/otlp/protobuf_include_suffix.h"
 
 #  include "opentelemetry/ext/http/client/http_client_factory.h"
-#  include "opentelemetry/ext/http/client/nosend/http_client_nosend.h"
 #  include "opentelemetry/ext/http/server/http_server.h"
 #  include "opentelemetry/sdk/trace/batch_span_processor.h"
+#  include "opentelemetry/sdk/trace/batch_span_processor_options.h"
 #  include "opentelemetry/sdk/trace/tracer_provider.h"
+#  include "opentelemetry/test_common/ext/http/client/http_client_test_factory.h"
+#  include "opentelemetry/test_common/ext/http/client/nosend/http_client_nosend.h"
 #  include "opentelemetry/trace/provider.h"
 
+#  include <google/protobuf/message_lite.h>
 #  include <gtest/gtest.h>
 #  include "gmock/gmock.h"
 
@@ -55,10 +58,18 @@ OtlpHttpClientOptions MakeOtlpHttpClientOptions(HttpRequestContentType content_t
   options.content_type  = content_type;
   options.console_debug = true;
   options.timeout       = std::chrono::system_clock::duration::zero();
-  options.http_headers.insert(
-      std::make_pair<const std::string, std::string>("Custom-Header-Key", "Custom-Header-Value"));
+  options.http_headers.insert(std::make_pair("Custom-Header-Key", "Custom-Header-Value"));
   OtlpHttpClientOptions otlp_http_client_options(
-      options.url, options.content_type, options.json_bytes_mapping, options.use_json_name,
+      options.url, false,                 /* ssl_insecure_skip_verify */
+      "", /* ssl_ca_cert_path */ "",      /* ssl_ca_cert_string */
+      "",                                 /* ssl_client_key_path */
+      "", /* ssl_client_key_string */ "", /* ssl_client_cert_path */
+      "",                                 /* ssl_client_cert_string */
+      "",                                 /* ssl_min_tls */
+      "",                                 /* ssl_max_tls */
+      "",                                 /* ssl_cipher */
+      "",                                 /* ssl_cipher_suite */
+      options.content_type, options.json_bytes_mapping, options.compression, options.use_json_name,
       options.console_debug, options.timeout, options.http_headers);
   if (!async_mode)
   {
@@ -86,7 +97,7 @@ public:
   static std::pair<OtlpHttpClient *, std::shared_ptr<http_client::HttpClient>>
   GetMockOtlpHttpClient(HttpRequestContentType content_type, bool async_mode = false)
   {
-    auto http_client = http_client::HttpClientFactory::CreateNoSend();
+    auto http_client = http_client::HttpClientTestFactory::Create();
     return {new OtlpHttpClient(MakeOtlpHttpClientOptions(content_type, async_mode), http_client),
             http_client};
   }
@@ -123,7 +134,7 @@ public:
 
     auto processor = std::unique_ptr<sdk::trace::SpanProcessor>(
         new sdk::trace::BatchSpanProcessor(std::move(exporter), processor_opts));
-    auto provider = nostd::shared_ptr<trace::TracerProvider>(
+    auto provider = nostd::shared_ptr<sdk::trace::TracerProvider>(
         new sdk::trace::TracerProvider(std::move(processor), resource));
 
     std::string report_trace_id;
@@ -146,32 +157,40 @@ public:
     auto mock_session =
         std::static_pointer_cast<http_client::nosend::Session>(no_send_client->session_);
     EXPECT_CALL(*mock_session, SendRequest)
-        .WillOnce([&mock_session, report_trace_id](
-                      std::shared_ptr<opentelemetry::ext::http::client::EventHandler> callback) {
-          auto check_json =
-              nlohmann::json::parse(mock_session->GetRequest()->body_, nullptr, false);
-          auto resource_span     = *check_json["resource_spans"].begin();
-          auto scope_span        = *resource_span["scope_spans"].begin();
-          auto span              = *scope_span["spans"].begin();
-          auto received_trace_id = span["trace_id"].get<std::string>();
-          EXPECT_EQ(received_trace_id, report_trace_id);
+        .WillOnce(
+            [&mock_session, report_trace_id](
+                const std::shared_ptr<opentelemetry::ext::http::client::EventHandler> &callback) {
+              auto check_json =
+                  nlohmann::json::parse(mock_session->GetRequest()->body_, nullptr, false);
+              auto resource_span     = *check_json["resourceSpans"].begin();
+              auto scope_span        = *resource_span["scopeSpans"].begin();
+              auto span              = *scope_span["spans"].begin();
+              auto received_trace_id = span["traceId"].get<std::string>();
+              EXPECT_EQ(received_trace_id, report_trace_id);
 
-          auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
-          ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
-          if (custom_header != mock_session->GetRequest()->headers_.end())
-          {
-            EXPECT_EQ("Custom-Header-Value", custom_header->second);
-          }
+              auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
+              ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
+              if (custom_header != mock_session->GetRequest()->headers_.end())
+              {
+                EXPECT_EQ("Custom-Header-Value", custom_header->second);
+              }
 
-          // let the otlp_http_client to continue
-          http_client::nosend::Response response;
-          response.Finish(*callback.get());
-        });
+              auto user_agent_header = mock_session->GetRequest()->headers_.find("User-Agent");
+              ASSERT_TRUE(user_agent_header != mock_session->GetRequest()->headers_.end());
+              if (user_agent_header != mock_session->GetRequest()->headers_.end())
+              {
+                EXPECT_EQ(GetOtlpDefaultUserAgent(), user_agent_header->second);
+              }
+
+              // let the otlp_http_client to continue
+              http_client::nosend::Response response;
+              response.Finish(*callback.get());
+            });
 
     child_span->End();
     parent_span->End();
 
-    static_cast<sdk::trace::TracerProvider *>(provider.get())->ForceFlush();
+    provider->ForceFlush();
   }
 
 #  ifdef ENABLE_ASYNC_EXPORT
@@ -207,7 +226,7 @@ public:
 
     auto processor = std::unique_ptr<sdk::trace::SpanProcessor>(
         new sdk::trace::BatchSpanProcessor(std::move(exporter), processor_opts));
-    auto provider = nostd::shared_ptr<trace::TracerProvider>(
+    auto provider = nostd::shared_ptr<sdk::trace::TracerProvider>(
         new sdk::trace::TracerProvider(std::move(processor), resource));
 
     std::string report_trace_id;
@@ -230,36 +249,44 @@ public:
     auto mock_session =
         std::static_pointer_cast<http_client::nosend::Session>(no_send_client->session_);
     EXPECT_CALL(*mock_session, SendRequest)
-        .WillOnce([&mock_session, report_trace_id](
-                      std::shared_ptr<opentelemetry::ext::http::client::EventHandler> callback) {
-          auto check_json =
-              nlohmann::json::parse(mock_session->GetRequest()->body_, nullptr, false);
-          auto resource_span     = *check_json["resource_spans"].begin();
-          auto scope_span        = *resource_span["scope_spans"].begin();
-          auto span              = *scope_span["spans"].begin();
-          auto received_trace_id = span["trace_id"].get<std::string>();
-          EXPECT_EQ(received_trace_id, report_trace_id);
+        .WillOnce(
+            [&mock_session, report_trace_id](
+                const std::shared_ptr<opentelemetry::ext::http::client::EventHandler> &callback) {
+              auto check_json =
+                  nlohmann::json::parse(mock_session->GetRequest()->body_, nullptr, false);
+              auto resource_span     = *check_json["resourceSpans"].begin();
+              auto scope_span        = *resource_span["scopeSpans"].begin();
+              auto span              = *scope_span["spans"].begin();
+              auto received_trace_id = span["traceId"].get<std::string>();
+              EXPECT_EQ(received_trace_id, report_trace_id);
 
-          auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
-          ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
-          if (custom_header != mock_session->GetRequest()->headers_.end())
-          {
-            EXPECT_EQ("Custom-Header-Value", custom_header->second);
-          }
+              auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
+              ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
+              if (custom_header != mock_session->GetRequest()->headers_.end())
+              {
+                EXPECT_EQ("Custom-Header-Value", custom_header->second);
+              }
 
-          // let the otlp_http_client to continue
-          std::thread async_finish{[callback]() {
-            std::this_thread::sleep_for(std::chrono::milliseconds(100));
-            http_client::nosend::Response response;
-            response.Finish(*callback.get());
-          }};
-          async_finish.detach();
-        });
+              auto user_agent_header = mock_session->GetRequest()->headers_.find("User-Agent");
+              ASSERT_TRUE(user_agent_header != mock_session->GetRequest()->headers_.end());
+              if (user_agent_header != mock_session->GetRequest()->headers_.end())
+              {
+                EXPECT_EQ(GetOtlpDefaultUserAgent(), user_agent_header->second);
+              }
+
+              // let the otlp_http_client to continue
+              std::thread async_finish{[callback]() {
+                std::this_thread::sleep_for(std::chrono::milliseconds(100));
+                http_client::nosend::Response response;
+                response.Finish(*callback.get());
+              }};
+              async_finish.detach();
+            });
 
     child_span->End();
     parent_span->End();
 
-    static_cast<sdk::trace::TracerProvider *>(provider.get())->ForceFlush();
+    provider->ForceFlush();
   }
 #  endif
 
@@ -295,7 +322,7 @@ public:
 
     auto processor = std::unique_ptr<sdk::trace::SpanProcessor>(
         new sdk::trace::BatchSpanProcessor(std::move(exporter), processor_opts));
-    auto provider = nostd::shared_ptr<trace::TracerProvider>(
+    auto provider = nostd::shared_ptr<sdk::trace::TracerProvider>(
         new sdk::trace::TracerProvider(std::move(processor), resource));
 
     std::string report_trace_id;
@@ -317,30 +344,32 @@ public:
     auto mock_session =
         std::static_pointer_cast<http_client::nosend::Session>(no_send_client->session_);
     EXPECT_CALL(*mock_session, SendRequest)
-        .WillOnce([&mock_session, report_trace_id](
-                      std::shared_ptr<opentelemetry::ext::http::client::EventHandler> callback) {
-          opentelemetry::proto::collector::trace::v1::ExportTraceServiceRequest request_body;
-          request_body.ParseFromArray(&mock_session->GetRequest()->body_[0],
-                                      static_cast<int>(mock_session->GetRequest()->body_.size()));
-          auto received_trace_id =
-              request_body.resource_spans(0).scope_spans(0).spans(0).trace_id();
-          EXPECT_EQ(received_trace_id, report_trace_id);
+        .WillOnce(
+            [&mock_session, report_trace_id](
+                const std::shared_ptr<opentelemetry::ext::http::client::EventHandler> &callback) {
+              opentelemetry::proto::collector::trace::v1::ExportTraceServiceRequest request_body;
+              request_body.ParseFromArray(
+                  &mock_session->GetRequest()->body_[0],
+                  static_cast<int>(mock_session->GetRequest()->body_.size()));
+              auto received_trace_id =
+                  request_body.resource_spans(0).scope_spans(0).spans(0).trace_id();
+              EXPECT_EQ(received_trace_id, report_trace_id);
 
-          auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
-          ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
-          if (custom_header != mock_session->GetRequest()->headers_.end())
-          {
-            EXPECT_EQ("Custom-Header-Value", custom_header->second);
-          }
+              auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
+              ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
+              if (custom_header != mock_session->GetRequest()->headers_.end())
+              {
+                EXPECT_EQ("Custom-Header-Value", custom_header->second);
+              }
 
-          http_client::nosend::Response response;
-          response.Finish(*callback.get());
-        });
+              http_client::nosend::Response response;
+              response.Finish(*callback.get());
+            });
 
     child_span->End();
     parent_span->End();
 
-    static_cast<sdk::trace::TracerProvider *>(provider.get())->ForceFlush();
+    provider->ForceFlush();
   }
 
 #  ifdef ENABLE_ASYNC_EXPORT
@@ -376,7 +405,7 @@ public:
 
     auto processor = std::unique_ptr<sdk::trace::SpanProcessor>(
         new sdk::trace::BatchSpanProcessor(std::move(exporter), processor_opts));
-    auto provider = nostd::shared_ptr<trace::TracerProvider>(
+    auto provider = nostd::shared_ptr<sdk::trace::TracerProvider>(
         new sdk::trace::TracerProvider(std::move(processor), resource));
 
     std::string report_trace_id;
@@ -398,35 +427,37 @@ public:
     auto mock_session =
         std::static_pointer_cast<http_client::nosend::Session>(no_send_client->session_);
     EXPECT_CALL(*mock_session, SendRequest)
-        .WillOnce([&mock_session, report_trace_id](
-                      std::shared_ptr<opentelemetry::ext::http::client::EventHandler> callback) {
-          opentelemetry::proto::collector::trace::v1::ExportTraceServiceRequest request_body;
-          request_body.ParseFromArray(&mock_session->GetRequest()->body_[0],
-                                      static_cast<int>(mock_session->GetRequest()->body_.size()));
-          auto received_trace_id =
-              request_body.resource_spans(0).scope_spans(0).spans(0).trace_id();
-          EXPECT_EQ(received_trace_id, report_trace_id);
+        .WillOnce(
+            [&mock_session, report_trace_id](
+                const std::shared_ptr<opentelemetry::ext::http::client::EventHandler> &callback) {
+              opentelemetry::proto::collector::trace::v1::ExportTraceServiceRequest request_body;
+              request_body.ParseFromArray(
+                  &mock_session->GetRequest()->body_[0],
+                  static_cast<int>(mock_session->GetRequest()->body_.size()));
+              auto received_trace_id =
+                  request_body.resource_spans(0).scope_spans(0).spans(0).trace_id();
+              EXPECT_EQ(received_trace_id, report_trace_id);
 
-          auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
-          ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
-          if (custom_header != mock_session->GetRequest()->headers_.end())
-          {
-            EXPECT_EQ("Custom-Header-Value", custom_header->second);
-          }
+              auto custom_header = mock_session->GetRequest()->headers_.find("Custom-Header-Key");
+              ASSERT_TRUE(custom_header != mock_session->GetRequest()->headers_.end());
+              if (custom_header != mock_session->GetRequest()->headers_.end())
+              {
+                EXPECT_EQ("Custom-Header-Value", custom_header->second);
+              }
 
-          // let the otlp_http_client to continue
-          std::thread async_finish{[callback]() {
-            std::this_thread::sleep_for(std::chrono::milliseconds(100));
-            http_client::nosend::Response response;
-            response.Finish(*callback.get());
-          }};
-          async_finish.detach();
-        });
+              // let the otlp_http_client to continue
+              std::thread async_finish{[callback]() {
+                std::this_thread::sleep_for(std::chrono::milliseconds(100));
+                http_client::nosend::Response response;
+                response.Finish(*callback.get());
+              }};
+              async_finish.detach();
+            });
 
     child_span->End();
     parent_span->End();
 
-    static_cast<sdk::trace::TracerProvider *>(provider.get())->ForceFlush();
+    provider->ForceFlush();
   }
 #  endif
 };
@@ -452,6 +483,7 @@ TEST_F(OtlpHttpExporterTestPeer, ExportJsonIntegrationTestSync)
 TEST_F(OtlpHttpExporterTestPeer, ExportJsonIntegrationTestAsync)
 {
   ExportJsonIntegrationTestAsync();
+  google::protobuf::ShutdownProtobufLibrary();
 }
 #  endif
 
@@ -495,6 +527,12 @@ TEST_F(OtlpHttpExporterTestPeer, ConfigJsonBytesMappingTest)
   EXPECT_EQ(GetOptions(exporter).json_bytes_mapping, JsonBytesMappingKind::kHex);
 }
 
+TEST(OtlpHttpExporterTest, ConfigDefaultProtocolTest)
+{
+  OtlpHttpExporterOptions opts;
+  EXPECT_EQ(opts.content_type, HttpRequestContentType::kBinary);
+}
+
 #  ifndef NO_GETENV
 // Test exporter configuration options with use_ssl_credentials
 TEST_F(OtlpHttpExporterTestPeer, ConfigFromEnv)
@@ -504,6 +542,7 @@ TEST_F(OtlpHttpExporterTestPeer, ConfigFromEnv)
   setenv("OTEL_EXPORTER_OTLP_TIMEOUT", "20s", 1);
   setenv("OTEL_EXPORTER_OTLP_HEADERS", "k1=v1,k2=v2", 1);
   setenv("OTEL_EXPORTER_OTLP_TRACES_HEADERS", "k1=v3,k1=v4", 1);
+  setenv("OTEL_EXPORTER_OTLP_PROTOCOL", "http/json", 1);
 
   std::unique_ptr<OtlpHttpExporter> exporter(new OtlpHttpExporter());
   EXPECT_EQ(GetOptions(exporter).url, url);
@@ -530,26 +569,29 @@ TEST_F(OtlpHttpExporterTestPeer, ConfigFromEnv)
     ++range.first;
     EXPECT_TRUE(range.first == range.second);
   }
+  EXPECT_EQ(GetOptions(exporter).content_type, HttpRequestContentType::kJson);
 
   unsetenv("OTEL_EXPORTER_OTLP_ENDPOINT");
   unsetenv("OTEL_EXPORTER_OTLP_TIMEOUT");
   unsetenv("OTEL_EXPORTER_OTLP_HEADERS");
   unsetenv("OTEL_EXPORTER_OTLP_TRACES_HEADERS");
+  unsetenv("OTEL_EXPORTER_OTLP_PROTOCOL");
 }
 
 TEST_F(OtlpHttpExporterTestPeer, ConfigFromTracesEnv)
 {
   const std::string url = "http://localhost:9999/v1/traces";
   setenv("OTEL_EXPORTER_OTLP_TRACES_ENDPOINT", url.c_str(), 1);
-  setenv("OTEL_EXPORTER_OTLP_TIMEOUT", "20s", 1);
+  setenv("OTEL_EXPORTER_OTLP_TRACES_TIMEOUT", "1eternity", 1);
   setenv("OTEL_EXPORTER_OTLP_HEADERS", "k1=v1,k2=v2", 1);
   setenv("OTEL_EXPORTER_OTLP_TRACES_HEADERS", "k1=v3,k1=v4", 1);
+  setenv("OTEL_EXPORTER_OTLP_TRACES_PROTOCOL", "http/json", 1);
 
   std::unique_ptr<OtlpHttpExporter> exporter(new OtlpHttpExporter());
   EXPECT_EQ(GetOptions(exporter).url, url);
   EXPECT_EQ(
       GetOptions(exporter).timeout.count(),
-      std::chrono::duration_cast<std::chrono::system_clock::duration>(std::chrono::seconds{20})
+      std::chrono::duration_cast<std::chrono::system_clock::duration>(std::chrono::seconds{10})
           .count());
   EXPECT_EQ(GetOptions(exporter).http_headers.size(), 3);
   {
@@ -570,15 +612,17 @@ TEST_F(OtlpHttpExporterTestPeer, ConfigFromTracesEnv)
     ++range.first;
     EXPECT_TRUE(range.first == range.second);
   }
+  EXPECT_EQ(GetOptions(exporter).content_type, HttpRequestContentType::kJson);
 
   unsetenv("OTEL_EXPORTER_OTLP_TRACES_ENDPOINT");
-  unsetenv("OTEL_EXPORTER_OTLP_TIMEOUT");
+  unsetenv("OTEL_EXPORTER_OTLP_TRACES_TIMEOUT");
   unsetenv("OTEL_EXPORTER_OTLP_HEADERS");
   unsetenv("OTEL_EXPORTER_OTLP_TRACES_HEADERS");
+  unsetenv("OTEL_EXPORTER_OTLP_TRACES_PROTOCOL");
 }
 #  endif
 
 }  // namespace otlp
 }  // namespace exporter
 OPENTELEMETRY_END_NAMESPACE
-#endif
+#endif /* OPENTELEMETRY_STL_VERSION */
