@@ -1,9 +1,34 @@
 // Copyright The OpenTelemetry Authors
 // SPDX-License-Identifier: Apache-2.0
 
-#include "opentelemetry/ext/http/client/curl/http_client_curl.h"
-
+#include <curl/curl.h>
+#include <curl/curlver.h>
+#include <zconf.h>
+#include <atomic>
+#include <chrono>
+#include <cstddef>
+#include <cstdint>
 #include <list>
+#include <mutex>
+#include <string>
+#include <thread>
+#include <unordered_map>
+#include <unordered_set>
+#include <utility>
+
+#include "opentelemetry/ext/http/client/curl/http_client_curl.h"
+#include "opentelemetry/ext/http/client/curl/http_operation_curl.h"
+#include "opentelemetry/ext/http/client/http_client.h"
+#include "opentelemetry/ext/http/common/url_parser.h"
+#include "opentelemetry/nostd/shared_ptr.h"
+#include "opentelemetry/nostd/string_view.h"
+#include "opentelemetry/version.h"
+
+#ifdef ENABLE_OTLP_COMPRESSION_PREVIEW
+#  include <zlib.h>
+#else
+#  include "opentelemetry/sdk/common/global_log_handler.h"
+#endif
 
 OPENTELEMETRY_BEGIN_NAMESPACE
 namespace ext
@@ -48,8 +73,52 @@ void Session::SendRequest(
     reuse_connection = session_id_ % http_client_.GetMaxSessionsPerConnection() != 0;
   }
 
-  curl_operation_.reset(new HttpOperation(http_request_->method_, url, callback_ptr,
-                                          http_request_->headers_, http_request_->body_, false,
+  if (http_request_->compression_ == opentelemetry::ext::http::client::Compression::kGzip)
+  {
+#ifdef ENABLE_OTLP_COMPRESSION_PREVIEW
+    http_request_->AddHeader("Content-Encoding", "gzip");
+
+    opentelemetry::ext::http::client::Body compressed_body(http_request_->body_.size());
+    z_stream zs;
+    zs.zalloc    = Z_NULL;
+    zs.zfree     = Z_NULL;
+    zs.opaque    = Z_NULL;
+    zs.avail_in  = static_cast<uInt>(http_request_->body_.size());
+    zs.next_in   = http_request_->body_.data();
+    zs.avail_out = static_cast<uInt>(compressed_body.size());
+    zs.next_out  = compressed_body.data();
+
+    // ZLIB: Have to maually specify 16 bits for the Gzip headers
+    const int window_bits = 15 + 16;
+
+    int stream =
+        deflateInit2(&zs, Z_DEFAULT_COMPRESSION, Z_DEFLATED, window_bits, 8, Z_DEFAULT_STRATEGY);
+
+    if (stream == Z_OK)
+    {
+      deflate(&zs, Z_FINISH);
+      deflateEnd(&zs);
+      compressed_body.resize(zs.total_out);
+      http_request_->SetBody(compressed_body);
+    }
+    else
+    {
+      if (callback)
+      {
+        callback->OnEvent(opentelemetry::ext::http::client::SessionState::CreateFailed, "");
+      }
+      is_session_active_.store(false, std::memory_order_release);
+    }
+#else
+    OTEL_INTERNAL_LOG_ERROR(
+        "[HTTP Client Curl] Set WITH_OTLP_HTTP_COMPRESSION=ON to use gzip compression with the "
+        "OTLP HTTP Exporter");
+#endif
+  }
+
+  curl_operation_.reset(new HttpOperation(http_request_->method_, url, http_request_->ssl_options_,
+                                          callback_ptr, http_request_->headers_,
+                                          http_request_->body_, http_request_->compression_, false,
                                           http_request_->timeout_ms_, reuse_connection));
   bool success =
       CURLE_OK == curl_operation_->SendAsync(this, [this, callback](HttpOperation &operation) {
@@ -116,13 +185,12 @@ void Session::FinishOperation()
 }
 
 HttpClient::HttpClient()
-    : next_session_id_{0},
+    : multi_handle_(curl_multi_init()),
+      next_session_id_{0},
       max_sessions_per_connection_{8},
       scheduled_delay_milliseconds_{std::chrono::milliseconds(256)},
       curl_global_initializer_(HttpCurlGlobalInitializer::GetInstance())
-{
-  multi_handle_ = curl_multi_init();
-}
+{}
 
 HttpClient::~HttpClient()
 {
@@ -240,6 +308,7 @@ void HttpClient::CleanupSession(uint64_t session_id)
     }
   }
 
+  bool need_wakeup_background_thread = false;
   {
     std::lock_guard<std::recursive_mutex> lock_guard{session_ids_m_};
     pending_to_add_session_ids_.erase(session_id);
@@ -253,10 +322,17 @@ void HttpClient::CleanupSession(uint64_t session_id)
       }
       else if (session->IsSessionActive() && session->GetOperation())
       {
-        session->FinishOperation();
+        // If this session is already running, give it to the background thread for cleanup.
+        pending_to_abort_sessions_[session_id] = std::move(session);
+        need_wakeup_background_thread          = true;
       }
     }
   }
+
+  if (need_wakeup_background_thread)
+  {
+    wakeupBackgroundThread();
+  }
 }
 
 void HttpClient::MaybeSpawnBackgroundThread()
@@ -386,7 +462,7 @@ void HttpClient::ScheduleAddSession(uint64_t session_id)
     std::lock_guard<std::recursive_mutex> lock_guard{session_ids_m_};
     pending_to_add_session_ids_.insert(session_id);
     pending_to_remove_session_handles_.erase(session_id);
-    pending_to_abort_session_ids_.erase(session_id);
+    pending_to_abort_sessions_.erase(session_id);
   }
 
   wakeupBackgroundThread();
@@ -395,9 +471,21 @@ void HttpClient::ScheduleAddSession(uint64_t session_id)
 void HttpClient::ScheduleAbortSession(uint64_t session_id)
 {
   {
-    std::lock_guard<std::recursive_mutex> lock_guard{session_ids_m_};
-    pending_to_abort_session_ids_.insert(session_id);
-    pending_to_add_session_ids_.erase(session_id);
+    std::lock_guard<std::mutex> lock_guard{sessions_m_};
+    auto session = sessions_.find(session_id);
+    if (session == sessions_.end())
+    {
+      std::lock_guard<std::recursive_mutex> lock_guard{session_ids_m_};
+      pending_to_add_session_ids_.erase(session_id);
+    }
+    else
+    {
+      std::lock_guard<std::recursive_mutex> lock_guard{session_ids_m_};
+      pending_to_abort_sessions_[session_id] = std::move(session->second);
+      pending_to_add_session_ids_.erase(session_id);
+
+      sessions_.erase(session);
+    }
   }
 
   wakeupBackgroundThread();
@@ -466,33 +554,23 @@ bool HttpClient::doAddSessions()
 
 bool HttpClient::doAbortSessions()
 {
-  std::list<std::shared_ptr<Session>> abort_sessions;
-  std::unordered_set<uint64_t> pending_to_abort_session_ids;
+  std::unordered_map<uint64_t, std::shared_ptr<Session>> pending_to_abort_sessions;
   {
     std::lock_guard<std::recursive_mutex> session_id_lock_guard{session_ids_m_};
-    pending_to_abort_session_ids_.swap(pending_to_abort_session_ids);
-  }
-
-  {
-    std::lock_guard<std::mutex> lock_guard{sessions_m_};
-    for (auto &session_id : pending_to_abort_session_ids)
-    {
-      auto session = sessions_.find(session_id);
-      if (session == sessions_.end())
-      {
-        continue;
-      }
-
-      abort_sessions.push_back(session->second);
-    }
+    pending_to_abort_sessions_.swap(pending_to_abort_sessions);
   }
 
   bool has_data = false;
-  for (auto session : abort_sessions)
+  for (const auto &session : pending_to_abort_sessions)
   {
-    if (session->GetOperation())
+    if (!session.second)
     {
-      session->FinishOperation();
+      continue;
+    }
+
+    if (session.second->GetOperation())
+    {
+      session.second->FinishOperation();
       has_data = true;
     }
   }
@@ -511,7 +589,8 @@ bool HttpClient::doRemoveSessions()
       std::lock_guard<std::recursive_mutex> session_id_lock_guard{session_ids_m_};
       pending_to_remove_session_handles_.swap(pending_to_remove_session_handles);
       pending_to_remove_sessions_.swap(pending_to_remove_sessions);
-
+    }
+    {
       // If user callback do not call CancelSession or FinishSession, We still need to remove it
       // from sessions_
       std::lock_guard<std::mutex> session_lock_guard{sessions_m_};
