@@ -1,11 +1,34 @@
 // Copyright The OpenTelemetry Authors
 // SPDX-License-Identifier: Apache-2.0
 
+#include <curl/curl.h>
+#include <curl/curlver.h>
+#include <curl/system.h>
+#include <algorithm>
+#include <atomic>
+#include <chrono>
+#include <cstdint>
 #include <cstring>
-
-#include "opentelemetry/ext/http/client/curl/http_operation_curl.h"
+#include <functional>
+#include <future>
+#include <istream>
+#include <map>
+#include <memory>
+#include <string>
+#include <thread>
+#include <utility>
+#include <vector>
 
 #include "opentelemetry/ext/http/client/curl/http_client_curl.h"
+#include "opentelemetry/ext/http/client/curl/http_operation_curl.h"
+#include "opentelemetry/ext/http/client/http_client.h"
+#include "opentelemetry/sdk/common/global_log_handler.h"
+#include "opentelemetry/version.h"
+
+// CURL_VERSION_BITS was added in CURL 7.43.0
+#ifndef CURL_VERSION_BITS
+#  define CURL_VERSION_BITS(x, y, z) ((x) << 16 | (y) << 8 | (z))
+#endif
 
 OPENTELEMETRY_BEGIN_NAMESPACE
 namespace ext
@@ -54,7 +77,7 @@ size_t HttpOperation::WriteVectorHeaderCallback(void *ptr, size_t size, size_t n
     return 0;
   }
 
-  const unsigned char *begin = (unsigned char *)(ptr);
+  const unsigned char *begin = static_cast<unsigned char *>(ptr);
   const unsigned char *end   = begin + size * nmemb;
   self->response_headers_.insert(self->response_headers_.end(), begin, end);
 
@@ -84,7 +107,7 @@ size_t HttpOperation::WriteVectorBodyCallback(void *ptr, size_t size, size_t nme
     return 0;
   }
 
-  const unsigned char *begin = (unsigned char *)(ptr);
+  const unsigned char *begin = static_cast<unsigned char *>(ptr);
   const unsigned char *end   = begin + size * nmemb;
   self->response_body_.insert(self->response_body_.end(), begin, end);
 
@@ -171,10 +194,10 @@ int HttpOperation::PreRequestCallback(void *clientp, char *, char *, int, int)
 
 #if LIBCURL_VERSION_NUM >= 0x072000
 int HttpOperation::OnProgressCallback(void *clientp,
-                                      curl_off_t dltotal,
-                                      curl_off_t dlnow,
-                                      curl_off_t ultotal,
-                                      curl_off_t ulnow)
+                                      curl_off_t /* dltotal */,
+                                      curl_off_t /* dlnow */,
+                                      curl_off_t /* ultotal */,
+                                      curl_off_t /* ulnow */)
 {
   HttpOperation *self = reinterpret_cast<HttpOperation *>(clientp);
   if (nullptr == self)
@@ -196,10 +219,10 @@ int HttpOperation::OnProgressCallback(void *clientp,
 }
 #else
 int HttpOperation::OnProgressCallback(void *clientp,
-                                      double dltotal,
-                                      double dlnow,
-                                      double ultotal,
-                                      double ulnow)
+                                      double /* dltotal */,
+                                      double /* dlnow */,
+                                      double /* ultotal */,
+                                      double /* ulnow */)
 {
   HttpOperation *self = reinterpret_cast<HttpOperation *>(clientp);
   if (nullptr == self)
@@ -217,7 +240,7 @@ int HttpOperation::OnProgressCallback(void *clientp,
 #endif
 
 void HttpOperation::DispatchEvent(opentelemetry::ext::http::client::SessionState type,
-                                  std::string reason)
+                                  const std::string &reason)
 {
   if (event_handle_ != nullptr)
   {
@@ -229,10 +252,12 @@ void HttpOperation::DispatchEvent(opentelemetry::ext::http::client::SessionState
 
 HttpOperation::HttpOperation(opentelemetry::ext::http::client::Method method,
                              std::string url,
+                             const HttpSslOptions &ssl_options,
                              opentelemetry::ext::http::client::EventHandler *event_handle,
                              // Default empty headers and empty request body
                              const opentelemetry::ext::http::client::Headers &request_headers,
                              const opentelemetry::ext::http::client::Body &request_body,
+                             const opentelemetry::ext::http::client::Compression &compression,
                              // Default connectivity and response size options
                              bool is_raw_response,
                              std::chrono::milliseconds http_conn_timeout,
@@ -248,12 +273,14 @@ HttpOperation::HttpOperation(opentelemetry::ext::http::client::Method method,
       last_curl_result_(CURLE_OK),
       event_handle_(event_handle),
       method_(method),
-      url_(url),
+      url_(std::move(url)),
+      ssl_options_(ssl_options),
       // Local vars
       request_headers_(request_headers),
       request_body_(request_body),
       request_nwrite_(0),
       session_state_(opentelemetry::ext::http::client::SessionState::Created),
+      compression_(compression),
       response_code_(0)
 {
   /* get a curl handle */
@@ -339,8 +366,8 @@ void HttpOperation::Cleanup()
     case opentelemetry::ext::http::client::SessionState::Connecting:
     case opentelemetry::ext::http::client::SessionState::Connected:
     case opentelemetry::ext::http::client::SessionState::Sending: {
-      DispatchEvent(opentelemetry::ext::http::client::SessionState::Cancelled,
-                    curl_easy_strerror(last_curl_result_));
+      const char *message = GetCurlErrorMessage(last_curl_result_);
+      DispatchEvent(opentelemetry::ext::http::client::SessionState::Cancelled, message);
       break;
     }
     default:
@@ -398,6 +425,150 @@ void HttpOperation::Cleanup()
   }
 }
 
+/*
+  Support for TLS min version, TLS max version.
+
+  To represent versions, the following symbols are needed:
+
+  Added in CURL 7.34.0:
+  - CURL_SSLVERSION_TLSv1_0 (do not use)
+  - CURL_SSLVERSION_TLSv1_1 (do not use)
+  - CURL_SSLVERSION_TLSv1_2
+
+  Added in CURL 7.52.0:
+  - CURL_SSLVERSION_TLSv1_3
+
+  Added in CURL 7.54.0:
+  - CURL_SSLVERSION_MAX_TLSv1_0 (do not use)
+  - CURL_SSLVERSION_MAX_TLSv1_1 (do not use)
+  - CURL_SSLVERSION_MAX_TLSv1_2
+  - CURL_SSLVERSION_MAX_TLSv1_3
+
+  For 7.34.0 <= CURL < 7.54.0,
+  we don't want to get into partial support.
+
+  CURL 7.54.0 is required.
+*/
+#if LIBCURL_VERSION_NUM >= CURL_VERSION_BITS(7, 54, 0)
+#  define HAVE_TLS_VERSION
+#endif
+
+static long parse_min_ssl_version(const std::string &version)
+{
+#ifdef HAVE_TLS_VERSION
+  if (version == "1.2")
+  {
+    return CURL_SSLVERSION_TLSv1_2;
+  }
+
+  if (version == "1.3")
+  {
+    return CURL_SSLVERSION_TLSv1_3;
+  }
+#endif
+
+  return 0;
+}
+
+static long parse_max_ssl_version(const std::string &version)
+{
+#ifdef HAVE_TLS_VERSION
+  if (version == "1.2")
+  {
+    return CURL_SSLVERSION_MAX_TLSv1_2;
+  }
+
+  if (version == "1.3")
+  {
+    return CURL_SSLVERSION_MAX_TLSv1_3;
+  }
+#endif
+
+  return 0;
+}
+
+const char *HttpOperation::GetCurlErrorMessage(CURLcode code)
+{
+  const char *message;
+
+  if (curl_error_message_[0] != '\0')
+  {
+    // Will contain contextual data
+    message = curl_error_message_;
+  }
+  else
+  {
+    // Generic message
+    message = curl_easy_strerror(code);
+  }
+
+  return message;
+}
+
+CURLcode HttpOperation::SetCurlPtrOption(CURLoption option, void *value)
+{
+  CURLcode rc;
+
+  /*
+    curl_easy_setopt() is a macro with variadic arguments, type unsafe.
+    Various SetCurlXxxOption() helpers ensure it is called with a pointer,
+    which can be:
+    - a string (const char*)
+    - a blob (struct curl_blob*)
+    - a headers chunk (curl_slist *)
+  */
+  rc = curl_easy_setopt(curl_resource_.easy_handle, option, value);
+
+  if (rc != CURLE_OK)
+  {
+    const char *message = GetCurlErrorMessage(rc);
+    OTEL_INTERNAL_LOG_ERROR("CURL, set option <" << std::to_string(option) << "> failed: <"
+                                                 << message << ">");
+  }
+
+  return rc;
+}
+
+CURLcode HttpOperation::SetCurlLongOption(CURLoption option, long value)
+{
+  CURLcode rc;
+
+  /*
+    curl_easy_setopt() is a macro with variadic arguments, type unsafe.
+    SetCurlLongOption() ensures it is called with a long.
+  */
+  rc = curl_easy_setopt(curl_resource_.easy_handle, option, value);
+
+  if (rc != CURLE_OK)
+  {
+    const char *message = GetCurlErrorMessage(rc);
+    OTEL_INTERNAL_LOG_ERROR("CURL, set option <" << std::to_string(option) << "> failed: <"
+                                                 << message << ">");
+  }
+
+  return rc;
+}
+
+CURLcode HttpOperation::SetCurlOffOption(CURLoption option, curl_off_t value)
+{
+  CURLcode rc;
+
+  /*
+    curl_easy_setopt() is a macro with variadic arguments, type unsafe.
+    SetCurlOffOption() ensures it is called with a curl_off_t.
+  */
+  rc = curl_easy_setopt(curl_resource_.easy_handle, option, value);
+
+  if (rc != CURLE_OK)
+  {
+    const char *message = GetCurlErrorMessage(rc);
+    OTEL_INTERNAL_LOG_ERROR("CURL, set option <" << std::to_string(option) << "> failed: <"
+                                                 << message << ">");
+  }
+
+  return rc;
+}
+
 CURLcode HttpOperation::Setup()
 {
   if (!curl_resource_.easy_handle)
@@ -405,54 +576,420 @@ CURLcode HttpOperation::Setup()
     return CURLE_FAILED_INIT;
   }
 
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_VERBOSE, 0);
+  CURLcode rc;
+
+  curl_error_message_[0] = '\0';
+  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_ERRORBUFFER, curl_error_message_);
+
+  rc = SetCurlLongOption(CURLOPT_VERBOSE, 0L);
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
 
   // Specify target URL
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_URL, url_.c_str());
+  rc = SetCurlStrOption(CURLOPT_URL, url_.c_str());
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
 
-  // TODO: support ssl cert verification for https request
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_SSL_VERIFYPEER, 0);  // 1L
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_SSL_VERIFYHOST, 0);  // 2L
+  if (ssl_options_.use_ssl)
+  {
+    /* 1 - CA CERT */
+
+    if (!ssl_options_.ssl_ca_cert_path.empty())
+    {
+      const char *path = ssl_options_.ssl_ca_cert_path.c_str();
+
+      rc = SetCurlStrOption(CURLOPT_CAINFO, path);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+    else if (!ssl_options_.ssl_ca_cert_string.empty())
+    {
+#if LIBCURL_VERSION_NUM >= CURL_VERSION_BITS(7, 77, 0)
+      const char *data = ssl_options_.ssl_ca_cert_string.c_str();
+      size_t data_len  = ssl_options_.ssl_ca_cert_string.length();
+
+      struct curl_blob stblob;
+      stblob.data  = const_cast<char *>(data);
+      stblob.len   = data_len;
+      stblob.flags = CURL_BLOB_COPY;
+
+      rc = SetCurlBlobOption(CURLOPT_CAINFO_BLOB, &stblob);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+#else
+      // CURL 7.77.0 required for CURLOPT_CAINFO_BLOB.
+      OTEL_INTERNAL_LOG_ERROR("CURL 7.77.0 required for CA CERT STRING");
+      return CURLE_UNKNOWN_OPTION;
+#endif
+    }
+
+    /* 2 - CLIENT KEY */
+
+    if (!ssl_options_.ssl_client_key_path.empty())
+    {
+      const char *path = ssl_options_.ssl_client_key_path.c_str();
+
+      rc = SetCurlStrOption(CURLOPT_SSLKEY, path);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlStrOption(CURLOPT_SSLKEYTYPE, "PEM");
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+    else if (!ssl_options_.ssl_client_key_string.empty())
+    {
+#if LIBCURL_VERSION_NUM >= CURL_VERSION_BITS(7, 71, 0)
+      const char *data = ssl_options_.ssl_client_key_string.c_str();
+      size_t data_len  = ssl_options_.ssl_client_key_string.length();
+
+      struct curl_blob stblob;
+      stblob.data  = const_cast<char *>(data);
+      stblob.len   = data_len;
+      stblob.flags = CURL_BLOB_COPY;
+
+      rc = SetCurlBlobOption(CURLOPT_SSLKEY_BLOB, &stblob);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlStrOption(CURLOPT_SSLKEYTYPE, "PEM");
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+#else
+      // CURL 7.71.0 required for CURLOPT_SSLKEY_BLOB.
+      OTEL_INTERNAL_LOG_ERROR("CURL 7.71.0 required for CLIENT KEY STRING");
+      return CURLE_UNKNOWN_OPTION;
+#endif
+    }
+
+    /* 3 - CLIENT CERT */
+
+    if (!ssl_options_.ssl_client_cert_path.empty())
+    {
+      const char *path = ssl_options_.ssl_client_cert_path.c_str();
+
+      rc = SetCurlStrOption(CURLOPT_SSLCERT, path);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlStrOption(CURLOPT_SSLCERTTYPE, "PEM");
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+    else if (!ssl_options_.ssl_client_cert_string.empty())
+    {
+#if LIBCURL_VERSION_NUM >= CURL_VERSION_BITS(7, 71, 0)
+      const char *data = ssl_options_.ssl_client_cert_string.c_str();
+      size_t data_len  = ssl_options_.ssl_client_cert_string.length();
+
+      struct curl_blob stblob;
+      stblob.data  = const_cast<char *>(data);
+      stblob.len   = data_len;
+      stblob.flags = CURL_BLOB_COPY;
+
+      rc = SetCurlBlobOption(CURLOPT_SSLCERT_BLOB, &stblob);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlStrOption(CURLOPT_SSLCERTTYPE, "PEM");
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+#else
+      // CURL 7.71.0 required for CURLOPT_SSLCERT_BLOB.
+      OTEL_INTERNAL_LOG_ERROR("CURL 7.71.0 required for CLIENT CERT STRING");
+      return CURLE_UNKNOWN_OPTION;
+#endif
+    }
+
+    /* 4 - TLS */
+
+#ifdef HAVE_TLS_VERSION
+    /* By default, TLSv1.2 or better is required (if we have TLS). */
+    long min_ssl_version = CURL_SSLVERSION_TLSv1_2;
+#else
+    long min_ssl_version = 0;
+#endif
+
+    if (!ssl_options_.ssl_min_tls.empty())
+    {
+#ifdef HAVE_TLS_VERSION
+      min_ssl_version = parse_min_ssl_version(ssl_options_.ssl_min_tls);
+
+      if (min_ssl_version == 0)
+      {
+        OTEL_INTERNAL_LOG_ERROR("Unknown min TLS version <" << ssl_options_.ssl_min_tls << ">");
+        return CURLE_UNKNOWN_OPTION;
+      }
+#else
+      OTEL_INTERNAL_LOG_ERROR("CURL 7.54.0 required for MIN TLS");
+      return CURLE_UNKNOWN_OPTION;
+#endif
+    }
+
+    /*
+     * Do not set a max TLS version by default.
+     * The CURL + openssl library may be more recent than this code,
+     * and support a version we do not know about.
+     */
+    long max_ssl_version = 0;
+
+    if (!ssl_options_.ssl_max_tls.empty())
+    {
+#ifdef HAVE_TLS_VERSION
+      max_ssl_version = parse_max_ssl_version(ssl_options_.ssl_max_tls);
+
+      if (max_ssl_version == 0)
+      {
+        OTEL_INTERNAL_LOG_ERROR("Unknown max TLS version <" << ssl_options_.ssl_max_tls << ">");
+        return CURLE_UNKNOWN_OPTION;
+      }
+#else
+      OTEL_INTERNAL_LOG_ERROR("CURL 7.54.0 required for MAX TLS");
+      return CURLE_UNKNOWN_OPTION;
+#endif
+    }
+
+    long version_range = min_ssl_version | max_ssl_version;
+    if (version_range != 0)
+    {
+      rc = SetCurlLongOption(CURLOPT_SSLVERSION, version_range);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+
+    /* 5 - CIPHER */
+
+    if (!ssl_options_.ssl_cipher.empty())
+    {
+      /* TLS 1.2 */
+      const char *cipher_list = ssl_options_.ssl_cipher.c_str();
+
+      rc = SetCurlStrOption(CURLOPT_SSL_CIPHER_LIST, cipher_list);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+
+    if (!ssl_options_.ssl_cipher_suite.empty())
+    {
+#if LIBCURL_VERSION_NUM >= CURL_VERSION_BITS(7, 61, 0)
+      /* TLS 1.3 */
+      const char *cipher_list = ssl_options_.ssl_cipher_suite.c_str();
+
+      rc = SetCurlStrOption(CURLOPT_TLS13_CIPHERS, cipher_list);
+#else
+      // CURL 7.61.0 required for CURLOPT_TLS13_CIPHERS.
+      OTEL_INTERNAL_LOG_ERROR("CURL 7.61.0 required for CIPHER SUITE");
+      return CURLE_UNKNOWN_OPTION;
+#endif
+
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+
+    if (ssl_options_.ssl_insecure_skip_verify)
+    {
+      /* 6 - DO NOT ENFORCE VERIFICATION, This is not secure. */
+      rc = SetCurlLongOption(CURLOPT_USE_SSL, static_cast<long>(CURLUSESSL_NONE));
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlLongOption(CURLOPT_SSL_VERIFYPEER, 0L);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlLongOption(CURLOPT_SSL_VERIFYHOST, 0L);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+    else
+    {
+      /* 6 - ENFORCE VERIFICATION */
+      rc = SetCurlLongOption(CURLOPT_USE_SSL, static_cast<long>(CURLUSESSL_ALL));
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlLongOption(CURLOPT_SSL_VERIFYPEER, 1L);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+
+      rc = SetCurlLongOption(CURLOPT_SSL_VERIFYHOST, 2L);
+      if (rc != CURLE_OK)
+      {
+        return rc;
+      }
+    }
+  }
+  else
+  {
+    rc = SetCurlLongOption(CURLOPT_SSL_VERIFYPEER, 0L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlLongOption(CURLOPT_SSL_VERIFYHOST, 0L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+  }
+
+  if (compression_ == opentelemetry::ext::http::client::Compression::kGzip)
+  {
+    rc = SetCurlStrOption(CURLOPT_ACCEPT_ENCODING, "gzip");
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+  }
 
   if (curl_resource_.headers_chunk != nullptr)
   {
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_HTTPHEADER, curl_resource_.headers_chunk);
+    rc = SetCurlListOption(CURLOPT_HTTPHEADER, curl_resource_.headers_chunk);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
   }
 
   // TODO: control local port to use
   // curl_easy_setopt(curl, CURLOPT_LOCALPORT, dcf_port);
 
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_TIMEOUT_MS, http_conn_timeout_.count());
+  rc = SetCurlLongOption(CURLOPT_TIMEOUT_MS, http_conn_timeout_.count());
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
 
   // abort if slower than 4kb/sec during 30 seconds
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_LOW_SPEED_TIME, 30L);
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_LOW_SPEED_LIMIT, 4096);
+  rc = SetCurlLongOption(CURLOPT_LOW_SPEED_TIME, 30L);
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
+
+  rc = SetCurlLongOption(CURLOPT_LOW_SPEED_LIMIT, 4096L);
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
+
   if (reuse_connection_)
   {
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_FRESH_CONNECT, 0L);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_FORBID_REUSE, 0L);
+    rc = SetCurlLongOption(CURLOPT_FRESH_CONNECT, 0L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlLongOption(CURLOPT_FORBID_REUSE, 0L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
   }
   else
   {
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_FRESH_CONNECT, 1L);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_FORBID_REUSE, 1L);
+    rc = SetCurlLongOption(CURLOPT_FRESH_CONNECT, 1L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+    rc = SetCurlLongOption(CURLOPT_FORBID_REUSE, 1L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
   }
 
   if (is_raw_response_)
   {
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_HEADER, true);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_WRITEFUNCTION,
-                     (void *)&HttpOperation::WriteMemoryCallback);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_WRITEDATA, (void *)this);
+    rc = SetCurlLongOption(CURLOPT_HEADER, 1L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlPtrOption(CURLOPT_WRITEFUNCTION,
+                          reinterpret_cast<void *>(&HttpOperation::WriteMemoryCallback));
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlPtrOption(CURLOPT_WRITEDATA, this);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
   }
   else
   {
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_WRITEFUNCTION,
-                     (void *)&HttpOperation::WriteVectorBodyCallback);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_WRITEDATA, (void *)this);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_HEADERFUNCTION,
-                     (void *)&HttpOperation::WriteVectorHeaderCallback);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_HEADERDATA, (void *)this);
+    rc = SetCurlPtrOption(CURLOPT_WRITEFUNCTION,
+                          reinterpret_cast<void *>(&HttpOperation::WriteVectorBodyCallback));
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlPtrOption(CURLOPT_WRITEDATA, this);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlPtrOption(CURLOPT_HEADERFUNCTION,
+                          reinterpret_cast<void *>(&HttpOperation::WriteVectorHeaderCallback));
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlPtrOption(CURLOPT_HEADERDATA, this);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
   }
 
   // TODO: only two methods supported for now - POST and GET
@@ -461,12 +998,36 @@ CURLcode HttpOperation::Setup()
     // Request buffer
     const curl_off_t req_size = static_cast<curl_off_t>(request_body_.size());
     // POST
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_POST, 1L);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_POSTFIELDS, NULL);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_POSTFIELDSIZE_LARGE, req_size);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_READFUNCTION,
-                     (void *)&HttpOperation::ReadMemoryCallback);
-    curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_READDATA, (void *)this);
+    rc = SetCurlLongOption(CURLOPT_POST, 1L);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlStrOption(CURLOPT_POSTFIELDS, NULL);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlOffOption(CURLOPT_POSTFIELDSIZE_LARGE, req_size);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlPtrOption(CURLOPT_READFUNCTION,
+                          reinterpret_cast<void *>(&HttpOperation::ReadMemoryCallback));
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
+
+    rc = SetCurlPtrOption(CURLOPT_READDATA, this);
+    if (rc != CURLE_OK)
+    {
+      return rc;
+    }
   }
   else if (method_ == opentelemetry::ext::http::client::Method::Get)
   {
@@ -474,23 +1035,50 @@ CURLcode HttpOperation::Setup()
   }
   else
   {
+    OTEL_INTERNAL_LOG_ERROR("Unexpected HTTP method");
     return CURLE_UNSUPPORTED_PROTOCOL;
   }
 
 #if LIBCURL_VERSION_NUM >= 0x072000
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_XFERINFOFUNCTION,
-                   (void *)&HttpOperation::OnProgressCallback);
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_XFERINFODATA, (void *)this);
+  rc = SetCurlPtrOption(CURLOPT_XFERINFOFUNCTION,
+                        reinterpret_cast<void *>(&HttpOperation::OnProgressCallback));
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
+
+  rc = SetCurlPtrOption(CURLOPT_XFERINFODATA, this);
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
 #else
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_PROGRESSFUNCTION,
-                   (void *)&HttpOperation::OnProgressCallback);
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_PROGRESSDATA, (void *)this);
+  rc = SetCurlPtrOption(CURLOPT_PROGRESSFUNCTION, (void *)&HttpOperation::OnProgressCallback);
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
+
+  rc = SetCurlPtrOption(CURLOPT_PROGRESSDATA, this);
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
 #endif
 
 #if LIBCURL_VERSION_NUM >= 0x075000
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_PREREQFUNCTION,
-                   (void *)&HttpOperation::PreRequestCallback);
-  curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_PREREQDATA, (void *)this);
+  rc = SetCurlPtrOption(CURLOPT_PREREQFUNCTION,
+                        reinterpret_cast<void *>(&HttpOperation::PreRequestCallback));
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
+
+  rc = SetCurlPtrOption(CURLOPT_PREREQDATA, this);
+  if (rc != CURLE_OK)
+  {
+    return rc;
+  }
 #endif
 
   return CURLE_OK;
@@ -509,8 +1097,8 @@ CURLcode HttpOperation::Send()
   last_curl_result_ = Setup();
   if (last_curl_result_ != CURLE_OK)
   {
-    DispatchEvent(opentelemetry::ext::http::client::SessionState::ConnectFailed,
-                  curl_easy_strerror(last_curl_result_));
+    const char *message = GetCurlErrorMessage(last_curl_result_);
+    DispatchEvent(opentelemetry::ext::http::client::SessionState::ConnectFailed, message);
     return last_curl_result_;
   }
 
@@ -553,6 +1141,8 @@ CURLcode HttpOperation::SendAsync(Session *session, std::function<void(HttpOpera
   last_curl_result_ = code;
   if (code != CURLE_OK)
   {
+    const char *message = GetCurlErrorMessage(code);
+    DispatchEvent(opentelemetry::ext::http::client::SessionState::ConnectFailed, message);
     return code;
   }
   curl_easy_setopt(curl_resource_.easy_handle, CURLOPT_PRIVATE, session);
@@ -581,7 +1171,8 @@ Headers HttpOperation::GetResponseHeaders()
     return result;
 
   std::stringstream ss;
-  std::string headers((const char *)&response_headers_[0], response_headers_.size());
+  std::string headers(reinterpret_cast<const char *>(&response_headers_[0]),
+                      response_headers_.size());
   ss.str(headers);
 
   std::string header;
@@ -634,8 +1225,9 @@ void HttpOperation::PerformCurlMessage(CURLcode code)
     switch (GetSessionState())
     {
       case opentelemetry::ext::http::client::SessionState::Connecting: {
+        const char *message = GetCurlErrorMessage(code);
         DispatchEvent(opentelemetry::ext::http::client::SessionState::ConnectFailed,
-                      curl_easy_strerror(code));  // couldn't connect - stage 1
+                      message);  // couldn't connect - stage 1
         break;
       }
       case opentelemetry::ext::http::client::SessionState::Connected:
@@ -645,8 +1237,8 @@ void HttpOperation::PerformCurlMessage(CURLcode code)
           DispatchEvent(opentelemetry::ext::http::client::SessionState::Sending);
         }
 
-        DispatchEvent(opentelemetry::ext::http::client::SessionState::SendFailed,
-                      curl_easy_strerror(code));
+        const char *message = GetCurlErrorMessage(code);
+        DispatchEvent(opentelemetry::ext::http::client::SessionState::SendFailed, message);
       }
       default:
         break;
