@@ -8,7 +8,7 @@
 #include "opentelemetry/sdk/trace/batch_span_processor_options.h"
 #include "opentelemetry/sdk/trace/exporter.h"
 #include "opentelemetry/sdk/trace/processor.h"
-#include "opentelemetry/sdk/version/version.h"
+#include "opentelemetry/version.h"
 
 OPENTELEMETRY_BEGIN_NAMESPACE
 namespace sdk
@@ -19,7 +19,7 @@ namespace trace
 /**
  * Factory class for BatchSpanProcessor.
  */
-class BatchSpanProcessorFactory
+class OPENTELEMETRY_EXPORT BatchSpanProcessorFactory
 {
 public:
   /**
