@@ -1,18 +1,33 @@
 // Copyright The OpenTelemetry Authors
 // SPDX-License-Identifier: Apache-2.0
 
-#ifndef ENABLE_METRICS_PREVIEW
-#  include "opentelemetry/sdk/metrics/async_instruments.h"
-#  include "opentelemetry/sdk/metrics/state/multi_metric_storage.h"
+#include <gtest/gtest.h>
+#include <map>
+#include <memory>
+#include <string>
+#include <utility>
 
-#  include <gtest/gtest.h>
+#include "opentelemetry/metrics/observer_result.h"
+#include "opentelemetry/nostd/utility.h"
+#include "opentelemetry/sdk/metrics/async_instruments.h"
+#include "opentelemetry/sdk/metrics/instruments.h"
+#include "opentelemetry/sdk/metrics/state/filtered_ordered_attribute_map.h"
+#include "opentelemetry/sdk/metrics/state/metric_storage.h"
+#include "opentelemetry/sdk/metrics/state/multi_metric_storage.h"
+#include "opentelemetry/sdk/metrics/state/observable_registry.h"
 
 using namespace opentelemetry;
 using namespace opentelemetry::sdk::metrics;
 
 using M = std::map<std::string, std::string>;
 
-void asyc_generate_measurements(opentelemetry::metrics::ObserverResult observer, void *state) {}
+namespace
+{
+// NOLINTNEXTLINE
+void asyc_generate_measurements(opentelemetry::metrics::ObserverResult /* observer */,
+                                void * /* state */)
+{}
+}  // namespace
 
 TEST(AsyncInstruments, ObservableInstrument)
 {
@@ -25,5 +40,3 @@ TEST(AsyncInstruments, ObservableInstrument)
                                                observable_registry);
   observable_counter_long.AddCallback(asyc_generate_measurements, nullptr);
 }
-
-#endif
