@@ -1,7 +1,8 @@
 // Copyright The OpenTelemetry Authors
 // SPDX-License-Identifier: Apache-2.0
 
-#ifndef ENABLE_METRICS_PREVIEW
+#ifdef ENABLE_METRICS_EXEMPLAR_PREVIEW
+
 #  include "opentelemetry/sdk/metrics/exemplar/no_exemplar_reservoir.h"
 #  include <gtest/gtest.h>
 
@@ -9,13 +10,13 @@ using namespace opentelemetry::sdk::metrics;
 
 TEST(NoExemplarReservoir, OfferMeasurement)
 {
-  auto reservoir = opentelemetry::sdk::metrics::NoExemplarReservoir::GetNoExemplarReservoir();
+  auto reservoir = opentelemetry::sdk::metrics::ExemplarReservoir::GetNoExemplarReservoir();
   reservoir->OfferMeasurement(1.0, MetricAttributes{}, opentelemetry::context::Context{},
                               std::chrono::system_clock::now());
-  reservoir->OfferMeasurement(1l, MetricAttributes{}, opentelemetry::context::Context{},
-                              std::chrono::system_clock::now());
+  reservoir->OfferMeasurement(static_cast<int64_t>(1), MetricAttributes{},
+                              opentelemetry::context::Context{}, std::chrono::system_clock::now());
   auto exemplar_data = reservoir->CollectAndReset(MetricAttributes{});
   ASSERT_TRUE(exemplar_data.empty());
 }
 
-#endif
+#endif  // ENABLE_METRICS_EXEMPLAR_PREVIEW
